/*
 * Copyright 2010 Google Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */
/*
 * This file was generated.
 *  with google-apis-code-generator 1.4.1 (build: 2013-08-01 15:32:38 UTC)
 *  on 2013-08-07 at 01:08:03 UTC 
 */

package com.google.api.services.sqladmin;

import com.google.api.client.googleapis.GoogleUtils;
import com.google.api.client.googleapis.services.AbstractGoogleClientRequest;
import com.google.api.client.googleapis.services.GoogleClientRequestInitializer;
import com.google.api.client.googleapis.services.json.AbstractGoogleJsonClient;
import com.google.api.client.http.HttpRequestInitializer;
import com.google.api.client.http.HttpTransport;
import com.google.api.client.json.JsonFactory;
import com.google.api.client.json.JsonObjectParser;
import com.google.common.base.Preconditions;

/**
 * Service definition for SQLAdmin (v1beta1).
 *
 * <p>
 * API for Cloud SQL database instance management.
 * </p>
 *
 * <p>
 * For more information about this service, see the
 * <a href="https://developers.google.com/cloud-sql/docs/admin-api/" target="_blank">API Documentation</a>
 * </p>
 *
 * <p>
 * This service uses {@link SQLAdminRequestInitializer} to initialize global parameters via its
 * {@link Builder}.
 * </p>
 *
 * <p>
 * Upgrade warning: this class now extends {@link AbstractGoogleJsonClient}, whereas in prior
 * version 1.8 it extended {@link com.google.api.client.googleapis.services.GoogleClient}.
 * </p>
 *
 * @since 1.3
 * @author Google, Inc.
 */
@SuppressWarnings("javadoc")
public class SQLAdmin extends AbstractGoogleJsonClient {

  // Note: Leave this static initializer at the top of the file.
  static {
    Preconditions.checkState(GoogleUtils.VERSION.equals("1.13.2-beta"),
        "You are currently running with version %s of google-api-client. " +
        "You need version 1.13.2-beta of google-api-client to run version " +
        "1.13.2-beta of the Cloud SQL Administration API library.", GoogleUtils.VERSION);
  }

  /**
   * The default encoded root URL of the service. This is determined when the library is generated
   * and normally should not be changed.
   *
   * @since 1.7
   */
  public static final String DEFAULT_ROOT_URL = "https://www.googleapis.com/";

  /**
   * The default encoded service path of the service. This is determined when the library is
   * generated and normally should not be changed.
   *
   * @since 1.7
   */
  public static final String DEFAULT_SERVICE_PATH = "sql/v1beta1/";

  /**
   * The default encoded base URL of the service. This is determined when the library is generated
   * and normally should not be changed.
   * @deprecated (scheduled to be removed in 1.13)
   */
  @Deprecated
  public static final String DEFAULT_BASE_URL = DEFAULT_ROOT_URL + DEFAULT_SERVICE_PATH;

  /**
   * Constructor.
   *
   * <p>
   * Use {@link Builder} if you need to specify any of the optional parameters.
   * </p>
   *
   * @param transport HTTP transport
   * @param jsonFactory JSON factory
   * @param httpRequestInitializer HTTP request initializer or {@code null} for none
   * @since 1.7
   */
  public SQLAdmin(HttpTransport transport, JsonFactory jsonFactory,
      HttpRequestInitializer httpRequestInitializer) {
    super(transport,
        jsonFactory,
        DEFAULT_ROOT_URL,
        DEFAULT_SERVICE_PATH,
        httpRequestInitializer,
        false);
  }

  /**
   * @param transport HTTP transport
   * @param httpRequestInitializer HTTP request initializer or {@code null} for none
   * @param rootUrl root URL of the service
   * @param servicePath service path
   * @param jsonObjectParser JSON object parser
   * @param googleClientRequestInitializer Google request initializer or {@code null} for none
   * @param applicationName application name to be sent in the User-Agent header of requests or
   *        {@code null} for none
   * @param suppressPatternChecks whether discovery pattern checks should be suppressed on required
   *        parameters
   */
  SQLAdmin(HttpTransport transport,
      HttpRequestInitializer httpRequestInitializer,
      String rootUrl,
      String servicePath,
      JsonObjectParser jsonObjectParser,
      GoogleClientRequestInitializer googleClientRequestInitializer,
      String applicationName,
      boolean suppressPatternChecks) {
    super(transport,
        httpRequestInitializer,
        rootUrl,
        servicePath,
        jsonObjectParser,
        googleClientRequestInitializer,
        applicationName,
        suppressPatternChecks);
  }

  @Override
  protected void initialize(AbstractGoogleClientRequest<?> httpClientRequest) throws java.io.IOException {
    super.initialize(httpClientRequest);
  }

  /**
   * An accessor for creating requests from the BackupRuns collection.
   *
   * <p>The typical use is:</p>
   * <pre>
   *   {@code SQLAdmin sqladmin = new SQLAdmin(...);}
   *   {@code SQLAdmin.BackupRuns.List request = sqladmin.backupRuns().list(parameters ...)}
   * </pre>
   *
   * @return the resource collection
   */
  public BackupRuns backupRuns() {
    return new BackupRuns();
  }

  /**
   * The "backupRuns" collection of methods.
   */
  public class BackupRuns {

    /**
     * Retrieves a resource containing information about a backup run.
     *
     * Create a request for the method "backupRuns.get".
     *
     * This request holds the parameters needed by the the sqladmin server.  After setting any optional
     * parameters, call the {@link Get#execute()} method to invoke the remote operation.
     *
     * @param project Project ID of the project that contains the instance. You can find this on the project summary page
     *        of the Google APIs Console.
     * @param instance Database instance ID. This does not include the project ID.
     * @param backupConfiguration Identifier for the backup configuration. This gets generated automatically when a backup
     *        configuration is created.
     * @param dueTime The time when this run is due to start in RFC 3339 format, for example 2012-11-15T16:19:00.094Z.
     * @return the request
     */
    public Get get(String project, String instance, String backupConfiguration, String dueTime) throws java.io.IOException {
      Get result = new Get(project, instance, backupConfiguration, dueTime);
      initialize(result);
      return result;
    }

    public class Get extends SQLAdminRequest<com.google.api.services.sqladmin.model.BackupRun> {

      private static final String REST_PATH = "projects/{project}/instances/{instance}/backupRuns/{backupConfiguration}";

      /**
       * Retrieves a resource containing information about a backup run.
       *
       * Create a request for the method "backupRuns.get".
       *
       * This request holds the parameters needed by the the sqladmin server.  After setting any
       * optional parameters, call the {@link Get#execute()} method to invoke the remote operation. <p>
       * {@link Get#initialize(AbstractGoogleClientRequest)} must be called to initialize this instance
       * immediately after invoking the constructor. </p>
       *
       * @param project Project ID of the project that contains the instance. You can find this on the project summary page
     *        of the Google APIs Console.
       * @param instance Database instance ID. This does not include the project ID.
       * @param backupConfiguration Identifier for the backup configuration. This gets generated automatically when a backup
     *        configuration is created.
       * @param dueTime The time when this run is due to start in RFC 3339 format, for example 2012-11-15T16:19:00.094Z.
       * @since 1.13
       */
      protected Get(String project, String instance, String backupConfiguration, String dueTime) {
        super(SQLAdmin.this, "GET", REST_PATH, null, com.google.api.services.sqladmin.model.BackupRun.class);
        this.project = Preconditions.checkNotNull(project, "Required parameter project must be specified.");
        this.instance = Preconditions.checkNotNull(instance, "Required parameter instance must be specified.");
        this.backupConfiguration = Preconditions.checkNotNull(backupConfiguration, "Required parameter backupConfiguration must be specified.");
        this.dueTime = Preconditions.checkNotNull(dueTime, "Required parameter dueTime must be specified.");
      }

      @Override
      public com.google.api.client.http.HttpResponse executeUsingHead() throws java.io.IOException {
        return super.executeUsingHead();
      }

      @Override
      public com.google.api.client.http.HttpRequest buildHttpRequestUsingHead() throws java.io.IOException {
        return super.buildHttpRequestUsingHead();
      }

      @Override
      public Get setAlt(String alt) {
        return (Get) super.setAlt(alt);
      }

      @Override
      public Get setFields(String fields) {
        return (Get) super.setFields(fields);
      }

      @Override
      public Get setKey(String key) {
        return (Get) super.setKey(key);
      }

      @Override
      public Get setOauthToken(String oauthToken) {
        return (Get) super.setOauthToken(oauthToken);
      }

      @Override
      public Get setPrettyPrint(Boolean prettyPrint) {
        return (Get) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Get setQuotaUser(String quotaUser) {
        return (Get) super.setQuotaUser(quotaUser);
      }

      @Override
      public Get setUserIp(String userIp) {
        return (Get) super.setUserIp(userIp);
      }

      /**
     * Project ID of the project that contains the instance. You can find this on the project
     * summary page of the Google APIs Console.
     */
      @com.google.api.client.util.Key
      private String project;

      /** Project ID of the project that contains the instance. You can find this on the project summary page
     of the Google APIs Console.
       */
      public String getProject() {
        return project;
      }

      /**
     * Project ID of the project that contains the instance. You can find this on the project
     * summary page of the Google APIs Console.
     */
      public Get setProject(String project) {
        this.project = project;
        return this;
      }

      /** Database instance ID. This does not include the project ID. */
      @com.google.api.client.util.Key
      private String instance;

      /** Database instance ID. This does not include the project ID.
       */
      public String getInstance() {
        return instance;
      }

      /** Database instance ID. This does not include the project ID. */
      public Get setInstance(String instance) {
        this.instance = instance;
        return this;
      }

      /**
     * Identifier for the backup configuration. This gets generated automatically when a backup
     * configuration is created.
     */
      @com.google.api.client.util.Key
      private String backupConfiguration;

      /** Identifier for the backup configuration. This gets generated automatically when a backup
     configuration is created.
       */
      public String getBackupConfiguration() {
        return backupConfiguration;
      }

      /**
     * Identifier for the backup configuration. This gets generated automatically when a backup
     * configuration is created.
     */
      public Get setBackupConfiguration(String backupConfiguration) {
        this.backupConfiguration = backupConfiguration;
        return this;
      }

      /**
     * The time when this run is due to start in RFC 3339 format, for example
     * 2012-11-15T16:19:00.094Z.
     */
      @com.google.api.client.util.Key
      private String dueTime;

      /** The time when this run is due to start in RFC 3339 format, for example 2012-11-15T16:19:00.094Z.
       */
      public String getDueTime() {
        return dueTime;
      }

      /**
     * The time when this run is due to start in RFC 3339 format, for example
     * 2012-11-15T16:19:00.094Z.
     */
      public Get setDueTime(String dueTime) {
        this.dueTime = dueTime;
        return this;
      }

    }
    /**
     * Lists all backup runs associated with a given instance and configuration in the reverse
     * chronological order of the enqueued time.
     *
     * Create a request for the method "backupRuns.list".
     *
     * This request holds the parameters needed by the the sqladmin server.  After setting any optional
     * parameters, call the {@link List#execute()} method to invoke the remote operation.
     *
     * @param project Project ID of the project that contains the instance. You can find this on the project summary page
     *        of the Google APIs Console.
     * @param instance Database instance ID. This does not include the project ID.
     * @param backupConfiguration Identifier for the backup configuration. This gets generated automatically when a backup
     *        configuration is created.
     * @return the request
     */
    public List list(String project, String instance, String backupConfiguration) throws java.io.IOException {
      List result = new List(project, instance, backupConfiguration);
      initialize(result);
      return result;
    }

    public class List extends SQLAdminRequest<com.google.api.services.sqladmin.model.BackupRunsListResponse> {

      private static final String REST_PATH = "projects/{project}/instances/{instance}/backupRuns";

      /**
       * Lists all backup runs associated with a given instance and configuration in the reverse
       * chronological order of the enqueued time.
       *
       * Create a request for the method "backupRuns.list".
       *
       * This request holds the parameters needed by the the sqladmin server.  After setting any
       * optional parameters, call the {@link List#execute()} method to invoke the remote operation. <p>
       * {@link List#initialize(AbstractGoogleClientRequest)} must be called to initialize this instance
       * immediately after invoking the constructor. </p>
       *
       * @param project Project ID of the project that contains the instance. You can find this on the project summary page
     *        of the Google APIs Console.
       * @param instance Database instance ID. This does not include the project ID.
       * @param backupConfiguration Identifier for the backup configuration. This gets generated automatically when a backup
     *        configuration is created.
       * @since 1.13
       */
      protected List(String project, String instance, String backupConfiguration) {
        super(SQLAdmin.this, "GET", REST_PATH, null, com.google.api.services.sqladmin.model.BackupRunsListResponse.class);
        this.project = Preconditions.checkNotNull(project, "Required parameter project must be specified.");
        this.instance = Preconditions.checkNotNull(instance, "Required parameter instance must be specified.");
        this.backupConfiguration = Preconditions.checkNotNull(backupConfiguration, "Required parameter backupConfiguration must be specified.");
      }

      @Override
      public com.google.api.client.http.HttpResponse executeUsingHead() throws java.io.IOException {
        return super.executeUsingHead();
      }

      @Override
      public com.google.api.client.http.HttpRequest buildHttpRequestUsingHead() throws java.io.IOException {
        return super.buildHttpRequestUsingHead();
      }

      @Override
      public List setAlt(String alt) {
        return (List) super.setAlt(alt);
      }

      @Override
      public List setFields(String fields) {
        return (List) super.setFields(fields);
      }

      @Override
      public List setKey(String key) {
        return (List) super.setKey(key);
      }

      @Override
      public List setOauthToken(String oauthToken) {
        return (List) super.setOauthToken(oauthToken);
      }

      @Override
      public List setPrettyPrint(Boolean prettyPrint) {
        return (List) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public List setQuotaUser(String quotaUser) {
        return (List) super.setQuotaUser(quotaUser);
      }

      @Override
      public List setUserIp(String userIp) {
        return (List) super.setUserIp(userIp);
      }

      /**
     * Project ID of the project that contains the instance. You can find this on the project
     * summary page of the Google APIs Console.
     */
      @com.google.api.client.util.Key
      private String project;

      /** Project ID of the project that contains the instance. You can find this on the project summary page
     of the Google APIs Console.
       */
      public String getProject() {
        return project;
      }

      /**
     * Project ID of the project that contains the instance. You can find this on the project
     * summary page of the Google APIs Console.
     */
      public List setProject(String project) {
        this.project = project;
        return this;
      }

      /** Database instance ID. This does not include the project ID. */
      @com.google.api.client.util.Key
      private String instance;

      /** Database instance ID. This does not include the project ID.
       */
      public String getInstance() {
        return instance;
      }

      /** Database instance ID. This does not include the project ID. */
      public List setInstance(String instance) {
        this.instance = instance;
        return this;
      }

      /**
     * Identifier for the backup configuration. This gets generated automatically when a backup
     * configuration is created.
     */
      @com.google.api.client.util.Key
      private String backupConfiguration;

      /** Identifier for the backup configuration. This gets generated automatically when a backup
     configuration is created.
       */
      public String getBackupConfiguration() {
        return backupConfiguration;
      }

      /**
     * Identifier for the backup configuration. This gets generated automatically when a backup
     * configuration is created.
     */
      public List setBackupConfiguration(String backupConfiguration) {
        this.backupConfiguration = backupConfiguration;
        return this;
      }

      /** A previously-returned page token representing part of the larger set of results to view. */
      @com.google.api.client.util.Key
      private String pageToken;

      /** A previously-returned page token representing part of the larger set of results to view.
       */
      public String getPageToken() {
        return pageToken;
      }

      /** A previously-returned page token representing part of the larger set of results to view. */
      public List setPageToken(String pageToken) {
        this.pageToken = pageToken;
        return this;
      }

      /** Maximum number of backup runs per response. */
      @com.google.api.client.util.Key
      private Integer maxResults;

      /** Maximum number of backup runs per response.
       */
      public Integer getMaxResults() {
        return maxResults;
      }

      /** Maximum number of backup runs per response. */
      public List setMaxResults(Integer maxResults) {
        this.maxResults = maxResults;
        return this;
      }

    }

  }

  /**
   * An accessor for creating requests from the Instances collection.
   *
   * <p>The typical use is:</p>
   * <pre>
   *   {@code SQLAdmin sqladmin = new SQLAdmin(...);}
   *   {@code SQLAdmin.Instances.List request = sqladmin.instances().list(parameters ...)}
   * </pre>
   *
   * @return the resource collection
   */
  public Instances instances() {
    return new Instances();
  }

  /**
   * The "instances" collection of methods.
   */
  public class Instances {

    /**
     * Deletes a database instance.
     *
     * Create a request for the method "instances.delete".
     *
     * This request holds the parameters needed by the the sqladmin server.  After setting any optional
     * parameters, call the {@link Delete#execute()} method to invoke the remote operation.
     *
     * @param project Project ID of the project that contains the instance to be deleted. You can find this on the project
     *        summary page of the Google APIs Console.
     * @param instance Database instance ID. This does not include the project ID.
     * @return the request
     */
    public Delete delete(String project, String instance) throws java.io.IOException {
      Delete result = new Delete(project, instance);
      initialize(result);
      return result;
    }

    public class Delete extends SQLAdminRequest<com.google.api.services.sqladmin.model.InstancesDeleteResponse> {

      private static final String REST_PATH = "projects/{project}/instances/{instance}";

      /**
       * Deletes a database instance.
       *
       * Create a request for the method "instances.delete".
       *
       * This request holds the parameters needed by the the sqladmin server.  After setting any
       * optional parameters, call the {@link Delete#execute()} method to invoke the remote operation.
       * <p> {@link Delete#initialize(AbstractGoogleClientRequest)} must be called to initialize this
       * instance immediately after invoking the constructor. </p>
       *
       * @param project Project ID of the project that contains the instance to be deleted. You can find this on the project
     *        summary page of the Google APIs Console.
       * @param instance Database instance ID. This does not include the project ID.
       * @since 1.13
       */
      protected Delete(String project, String instance) {
        super(SQLAdmin.this, "DELETE", REST_PATH, null, com.google.api.services.sqladmin.model.InstancesDeleteResponse.class);
        this.project = Preconditions.checkNotNull(project, "Required parameter project must be specified.");
        this.instance = Preconditions.checkNotNull(instance, "Required parameter instance must be specified.");
      }

      @Override
      public Delete setAlt(String alt) {
        return (Delete) super.setAlt(alt);
      }

      @Override
      public Delete setFields(String fields) {
        return (Delete) super.setFields(fields);
      }

      @Override
      public Delete setKey(String key) {
        return (Delete) super.setKey(key);
      }

      @Override
      public Delete setOauthToken(String oauthToken) {
        return (Delete) super.setOauthToken(oauthToken);
      }

      @Override
      public Delete setPrettyPrint(Boolean prettyPrint) {
        return (Delete) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Delete setQuotaUser(String quotaUser) {
        return (Delete) super.setQuotaUser(quotaUser);
      }

      @Override
      public Delete setUserIp(String userIp) {
        return (Delete) super.setUserIp(userIp);
      }

      /**
     * Project ID of the project that contains the instance to be deleted. You can find this on the
     * project summary page of the Google APIs Console.
     */
      @com.google.api.client.util.Key
      private String project;

      /** Project ID of the project that contains the instance to be deleted. You can find this on the
     project summary page of the Google APIs Console.
       */
      public String getProject() {
        return project;
      }

      /**
     * Project ID of the project that contains the instance to be deleted. You can find this on the
     * project summary page of the Google APIs Console.
     */
      public Delete setProject(String project) {
        this.project = project;
        return this;
      }

      /** Database instance ID. This does not include the project ID. */
      @com.google.api.client.util.Key
      private String instance;

      /** Database instance ID. This does not include the project ID.
       */
      public String getInstance() {
        return instance;
      }

      /** Database instance ID. This does not include the project ID. */
      public Delete setInstance(String instance) {
        this.instance = instance;
        return this;
      }

    }
    /**
     * Exports data from a database instance to a Google Cloud Storage bucket as a MySQL dump file.
     *
     * Create a request for the method "instances.export".
     *
     * This request holds the parameters needed by the the sqladmin server.  After setting any optional
     * parameters, call the {@link Export#execute()} method to invoke the remote operation.
     *
     * @param project Project ID of the project that contains the instance to be exported. You can find this on the
     *        project summary page of the Google APIs Console.
     * @param instance Database instance ID. This does not include the project ID.
     * @param content the {@link com.google.api.services.sqladmin.model.InstancesExportRequest}
     * @return the request
     */
    public Export export(String project, String instance, com.google.api.services.sqladmin.model.InstancesExportRequest content) throws java.io.IOException {
      Export result = new Export(project, instance, content);
      initialize(result);
      return result;
    }

    public class Export extends SQLAdminRequest<com.google.api.services.sqladmin.model.InstancesExportResponse> {

      private static final String REST_PATH = "projects/{project}/instances/{instance}/export";

      /**
       * Exports data from a database instance to a Google Cloud Storage bucket as a MySQL dump file.
       *
       * Create a request for the method "instances.export".
       *
       * This request holds the parameters needed by the the sqladmin server.  After setting any
       * optional parameters, call the {@link Export#execute()} method to invoke the remote operation.
       * <p> {@link Export#initialize(AbstractGoogleClientRequest)} must be called to initialize this
       * instance immediately after invoking the constructor. </p>
       *
       * @param project Project ID of the project that contains the instance to be exported. You can find this on the
     *        project summary page of the Google APIs Console.
       * @param instance Database instance ID. This does not include the project ID.
       * @param content the {@link com.google.api.services.sqladmin.model.InstancesExportRequest}
       * @since 1.13
       */
      protected Export(String project, String instance, com.google.api.services.sqladmin.model.InstancesExportRequest content) {
        super(SQLAdmin.this, "POST", REST_PATH, content, com.google.api.services.sqladmin.model.InstancesExportResponse.class);
        this.project = Preconditions.checkNotNull(project, "Required parameter project must be specified.");
        this.instance = Preconditions.checkNotNull(instance, "Required parameter instance must be specified.");
      }

      @Override
      public Export setAlt(String alt) {
        return (Export) super.setAlt(alt);
      }

      @Override
      public Export setFields(String fields) {
        return (Export) super.setFields(fields);
      }

      @Override
      public Export setKey(String key) {
        return (Export) super.setKey(key);
      }

      @Override
      public Export setOauthToken(String oauthToken) {
        return (Export) super.setOauthToken(oauthToken);
      }

      @Override
      public Export setPrettyPrint(Boolean prettyPrint) {
        return (Export) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Export setQuotaUser(String quotaUser) {
        return (Export) super.setQuotaUser(quotaUser);
      }

      @Override
      public Export setUserIp(String userIp) {
        return (Export) super.setUserIp(userIp);
      }

      /**
     * Project ID of the project that contains the instance to be exported. You can find this on the
     * project summary page of the Google APIs Console.
     */
      @com.google.api.client.util.Key
      private String project;

      /** Project ID of the project that contains the instance to be exported. You can find this on the
     project summary page of the Google APIs Console.
       */
      public String getProject() {
        return project;
      }

      /**
     * Project ID of the project that contains the instance to be exported. You can find this on the
     * project summary page of the Google APIs Console.
     */
      public Export setProject(String project) {
        this.project = project;
        return this;
      }

      /** Database instance ID. This does not include the project ID. */
      @com.google.api.client.util.Key
      private String instance;

      /** Database instance ID. This does not include the project ID.
       */
      public String getInstance() {
        return instance;
      }

      /** Database instance ID. This does not include the project ID. */
      public Export setInstance(String instance) {
        this.instance = instance;
        return this;
      }

    }
    /**
     * Retrieves a resource containing information about a database instance.
     *
     * Create a request for the method "instances.get".
     *
     * This request holds the parameters needed by the the sqladmin server.  After setting any optional
     * parameters, call the {@link Get#execute()} method to invoke the remote operation.
     *
     * @param project Project ID of the project that contains the instance. You can find this on the project summary page
     *        of the Google APIs Console.
     * @param instance Database instance ID. This does not include the project ID.
     * @return the request
     */
    public Get get(String project, String instance) throws java.io.IOException {
      Get result = new Get(project, instance);
      initialize(result);
      return result;
    }

    public class Get extends SQLAdminRequest<com.google.api.services.sqladmin.model.DatabaseInstance> {

      private static final String REST_PATH = "projects/{project}/instances/{instance}";

      /**
       * Retrieves a resource containing information about a database instance.
       *
       * Create a request for the method "instances.get".
       *
       * This request holds the parameters needed by the the sqladmin server.  After setting any
       * optional parameters, call the {@link Get#execute()} method to invoke the remote operation. <p>
       * {@link Get#initialize(AbstractGoogleClientRequest)} must be called to initialize this instance
       * immediately after invoking the constructor. </p>
       *
       * @param project Project ID of the project that contains the instance. You can find this on the project summary page
     *        of the Google APIs Console.
       * @param instance Database instance ID. This does not include the project ID.
       * @since 1.13
       */
      protected Get(String project, String instance) {
        super(SQLAdmin.this, "GET", REST_PATH, null, com.google.api.services.sqladmin.model.DatabaseInstance.class);
        this.project = Preconditions.checkNotNull(project, "Required parameter project must be specified.");
        this.instance = Preconditions.checkNotNull(instance, "Required parameter instance must be specified.");
      }

      @Override
      public com.google.api.client.http.HttpResponse executeUsingHead() throws java.io.IOException {
        return super.executeUsingHead();
      }

      @Override
      public com.google.api.client.http.HttpRequest buildHttpRequestUsingHead() throws java.io.IOException {
        return super.buildHttpRequestUsingHead();
      }

      @Override
      public Get setAlt(String alt) {
        return (Get) super.setAlt(alt);
      }

      @Override
      public Get setFields(String fields) {
        return (Get) super.setFields(fields);
      }

      @Override
      public Get setKey(String key) {
        return (Get) super.setKey(key);
      }

      @Override
      public Get setOauthToken(String oauthToken) {
        return (Get) super.setOauthToken(oauthToken);
      }

      @Override
      public Get setPrettyPrint(Boolean prettyPrint) {
        return (Get) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Get setQuotaUser(String quotaUser) {
        return (Get) super.setQuotaUser(quotaUser);
      }

      @Override
      public Get setUserIp(String userIp) {
        return (Get) super.setUserIp(userIp);
      }

      /**
     * Project ID of the project that contains the instance. You can find this on the project
     * summary page of the Google APIs Console.
     */
      @com.google.api.client.util.Key
      private String project;

      /** Project ID of the project that contains the instance. You can find this on the project summary page
     of the Google APIs Console.
       */
      public String getProject() {
        return project;
      }

      /**
     * Project ID of the project that contains the instance. You can find this on the project
     * summary page of the Google APIs Console.
     */
      public Get setProject(String project) {
        this.project = project;
        return this;
      }

      /** Database instance ID. This does not include the project ID. */
      @com.google.api.client.util.Key
      private String instance;

      /** Database instance ID. This does not include the project ID.
       */
      public String getInstance() {
        return instance;
      }

      /** Database instance ID. This does not include the project ID. */
      public Get setInstance(String instance) {
        this.instance = instance;
        return this;
      }

    }
    /**
     * Imports data into a database instance from a MySQL dump file in Google Cloud Storage.
     *
     * Create a request for the method "instances.import".
     *
     * This request holds the parameters needed by the the sqladmin server.  After setting any optional
     * parameters, call the {@link SQLAdminImport#execute()} method to invoke the remote operation.
     *
     * @param project Project ID of the project that contains the instance. You can find this on the project summary page
     *        of the Google APIs Console.
     * @param instance Database instance ID. This does not include the project ID.
     * @param content the {@link com.google.api.services.sqladmin.model.InstancesImportRequest}
     * @return the request
     */
    public SQLAdminImport sqladminImport(String project, String instance, com.google.api.services.sqladmin.model.InstancesImportRequest content) throws java.io.IOException {
      SQLAdminImport result = new SQLAdminImport(project, instance, content);
      initialize(result);
      return result;
    }

    public class SQLAdminImport extends SQLAdminRequest<com.google.api.services.sqladmin.model.InstancesImportResponse> {

      private static final String REST_PATH = "projects/{project}/instances/{instance}/import";

      /**
       * Imports data into a database instance from a MySQL dump file in Google Cloud Storage.
       *
       * Create a request for the method "instances.import".
       *
       * This request holds the parameters needed by the the sqladmin server.  After setting any
       * optional parameters, call the {@link SQLAdminImport#execute()} method to invoke the remote
       * operation. <p> {@link SQLAdminImport#initialize(AbstractGoogleClientRequest)} must be called to
       * initialize this instance immediately after invoking the constructor. </p>
       *
       * @param project Project ID of the project that contains the instance. You can find this on the project summary page
     *        of the Google APIs Console.
       * @param instance Database instance ID. This does not include the project ID.
       * @param content the {@link com.google.api.services.sqladmin.model.InstancesImportRequest}
       * @since 1.13
       */
      protected SQLAdminImport(String project, String instance, com.google.api.services.sqladmin.model.InstancesImportRequest content) {
        super(SQLAdmin.this, "POST", REST_PATH, content, com.google.api.services.sqladmin.model.InstancesImportResponse.class);
        this.project = Preconditions.checkNotNull(project, "Required parameter project must be specified.");
        this.instance = Preconditions.checkNotNull(instance, "Required parameter instance must be specified.");
      }

      @Override
      public SQLAdminImport setAlt(String alt) {
        return (SQLAdminImport) super.setAlt(alt);
      }

      @Override
      public SQLAdminImport setFields(String fields) {
        return (SQLAdminImport) super.setFields(fields);
      }

      @Override
      public SQLAdminImport setKey(String key) {
        return (SQLAdminImport) super.setKey(key);
      }

      @Override
      public SQLAdminImport setOauthToken(String oauthToken) {
        return (SQLAdminImport) super.setOauthToken(oauthToken);
      }

      @Override
      public SQLAdminImport setPrettyPrint(Boolean prettyPrint) {
        return (SQLAdminImport) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public SQLAdminImport setQuotaUser(String quotaUser) {
        return (SQLAdminImport) super.setQuotaUser(quotaUser);
      }

      @Override
      public SQLAdminImport setUserIp(String userIp) {
        return (SQLAdminImport) super.setUserIp(userIp);
      }

      /**
     * Project ID of the project that contains the instance. You can find this on the project
     * summary page of the Google APIs Console.
     */
      @com.google.api.client.util.Key
      private String project;

      /** Project ID of the project that contains the instance. You can find this on the project summary page
     of the Google APIs Console.
       */
      public String getProject() {
        return project;
      }

      /**
     * Project ID of the project that contains the instance. You can find this on the project
     * summary page of the Google APIs Console.
     */
      public SQLAdminImport setProject(String project) {
        this.project = project;
        return this;
      }

      /** Database instance ID. This does not include the project ID. */
      @com.google.api.client.util.Key
      private String instance;

      /** Database instance ID. This does not include the project ID.
       */
      public String getInstance() {
        return instance;
      }

      /** Database instance ID. This does not include the project ID. */
      public SQLAdminImport setInstance(String instance) {
        this.instance = instance;
        return this;
      }

    }
    /**
     * Creates a new database instance.
     *
     * Create a request for the method "instances.insert".
     *
     * This request holds the parameters needed by the the sqladmin server.  After setting any optional
     * parameters, call the {@link Insert#execute()} method to invoke the remote operation.
     *
     * @param project Project ID of the project to which the newly created database instances should belong. You can find
     *        this on the project summary page of the Google APIs Console.
     * @param content the {@link com.google.api.services.sqladmin.model.DatabaseInstance}
     * @return the request
     */
    public Insert insert(String project, com.google.api.services.sqladmin.model.DatabaseInstance content) throws java.io.IOException {
      Insert result = new Insert(project, content);
      initialize(result);
      return result;
    }

    public class Insert extends SQLAdminRequest<com.google.api.services.sqladmin.model.InstancesInsertResponse> {

      private static final String REST_PATH = "projects/{project}/instances";

      /**
       * Creates a new database instance.
       *
       * Create a request for the method "instances.insert".
       *
       * This request holds the parameters needed by the the sqladmin server.  After setting any
       * optional parameters, call the {@link Insert#execute()} method to invoke the remote operation.
       * <p> {@link Insert#initialize(AbstractGoogleClientRequest)} must be called to initialize this
       * instance immediately after invoking the constructor. </p>
       *
       * @param project Project ID of the project to which the newly created database instances should belong. You can find
     *        this on the project summary page of the Google APIs Console.
       * @param content the {@link com.google.api.services.sqladmin.model.DatabaseInstance}
       * @since 1.13
       */
      protected Insert(String project, com.google.api.services.sqladmin.model.DatabaseInstance content) {
        super(SQLAdmin.this, "POST", REST_PATH, content, com.google.api.services.sqladmin.model.InstancesInsertResponse.class);
        this.project = Preconditions.checkNotNull(project, "Required parameter project must be specified.");
      }

      @Override
      public Insert setAlt(String alt) {
        return (Insert) super.setAlt(alt);
      }

      @Override
      public Insert setFields(String fields) {
        return (Insert) super.setFields(fields);
      }

      @Override
      public Insert setKey(String key) {
        return (Insert) super.setKey(key);
      }

      @Override
      public Insert setOauthToken(String oauthToken) {
        return (Insert) super.setOauthToken(oauthToken);
      }

      @Override
      public Insert setPrettyPrint(Boolean prettyPrint) {
        return (Insert) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Insert setQuotaUser(String quotaUser) {
        return (Insert) super.setQuotaUser(quotaUser);
      }

      @Override
      public Insert setUserIp(String userIp) {
        return (Insert) super.setUserIp(userIp);
      }

      /**
     * Project ID of the project to which the newly created database instances should belong. You
     * can find this on the project summary page of the Google APIs Console.
     */
      @com.google.api.client.util.Key
      private String project;

      /** Project ID of the project to which the newly created database instances should belong. You can find
     this on the project summary page of the Google APIs Console.
       */
      public String getProject() {
        return project;
      }

      /**
     * Project ID of the project to which the newly created database instances should belong. You
     * can find this on the project summary page of the Google APIs Console.
     */
      public Insert setProject(String project) {
        this.project = project;
        return this;
      }

    }
    /**
     * Lists instances under a given project in the alphabetical order of the instance name.
     *
     * Create a request for the method "instances.list".
     *
     * This request holds the parameters needed by the the sqladmin server.  After setting any optional
     * parameters, call the {@link List#execute()} method to invoke the remote operation.
     *
     * @param project Project ID of the project for which to list database instances. You can find this on the project
     *        summary page of the Google APIs Console.
     * @return the request
     */
    public List list(String project) throws java.io.IOException {
      List result = new List(project);
      initialize(result);
      return result;
    }

    public class List extends SQLAdminRequest<com.google.api.services.sqladmin.model.InstancesListResponse> {

      private static final String REST_PATH = "projects/{project}/instances";

      /**
       * Lists instances under a given project in the alphabetical order of the instance name.
       *
       * Create a request for the method "instances.list".
       *
       * This request holds the parameters needed by the the sqladmin server.  After setting any
       * optional parameters, call the {@link List#execute()} method to invoke the remote operation. <p>
       * {@link List#initialize(AbstractGoogleClientRequest)} must be called to initialize this instance
       * immediately after invoking the constructor. </p>
       *
       * @param project Project ID of the project for which to list database instances. You can find this on the project
     *        summary page of the Google APIs Console.
       * @since 1.13
       */
      protected List(String project) {
        super(SQLAdmin.this, "GET", REST_PATH, null, com.google.api.services.sqladmin.model.InstancesListResponse.class);
        this.project = Preconditions.checkNotNull(project, "Required parameter project must be specified.");
      }

      @Override
      public com.google.api.client.http.HttpResponse executeUsingHead() throws java.io.IOException {
        return super.executeUsingHead();
      }

      @Override
      public com.google.api.client.http.HttpRequest buildHttpRequestUsingHead() throws java.io.IOException {
        return super.buildHttpRequestUsingHead();
      }

      @Override
      public List setAlt(String alt) {
        return (List) super.setAlt(alt);
      }

      @Override
      public List setFields(String fields) {
        return (List) super.setFields(fields);
      }

      @Override
      public List setKey(String key) {
        return (List) super.setKey(key);
      }

      @Override
      public List setOauthToken(String oauthToken) {
        return (List) super.setOauthToken(oauthToken);
      }

      @Override
      public List setPrettyPrint(Boolean prettyPrint) {
        return (List) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public List setQuotaUser(String quotaUser) {
        return (List) super.setQuotaUser(quotaUser);
      }

      @Override
      public List setUserIp(String userIp) {
        return (List) super.setUserIp(userIp);
      }

      /**
     * Project ID of the project for which to list database instances. You can find this on the
     * project summary page of the Google APIs Console.
     */
      @com.google.api.client.util.Key
      private String project;

      /** Project ID of the project for which to list database instances. You can find this on the project
     summary page of the Google APIs Console.
       */
      public String getProject() {
        return project;
      }

      /**
     * Project ID of the project for which to list database instances. You can find this on the
     * project summary page of the Google APIs Console.
     */
      public List setProject(String project) {
        this.project = project;
        return this;
      }

      /** A previously-returned page token representing part of the larger set of results to view. */
      @com.google.api.client.util.Key
      private String pageToken;

      /** A previously-returned page token representing part of the larger set of results to view.
       */
      public String getPageToken() {
        return pageToken;
      }

      /** A previously-returned page token representing part of the larger set of results to view. */
      public List setPageToken(String pageToken) {
        this.pageToken = pageToken;
        return this;
      }

      /** The maximum number of results to return per response. */
      @com.google.api.client.util.Key
      private Long maxResults;

      /** The maximum number of results to return per response.
       */
      public Long getMaxResults() {
        return maxResults;
      }

      /** The maximum number of results to return per response. */
      public List setMaxResults(Long maxResults) {
        this.maxResults = maxResults;
        return this;
      }

    }
    /**
     * Updates settings of a database instance. Caution: This is not a partial update, so you must
     * include values for all the settings that you want to retain. For partial updates, use patch..
     * This method supports patch semantics.
     *
     * Create a request for the method "instances.patch".
     *
     * This request holds the parameters needed by the the sqladmin server.  After setting any optional
     * parameters, call the {@link Patch#execute()} method to invoke the remote operation.
     *
     * @param project Project ID of the project that contains the instance. You can find this on the project summary page
     *        of the Google APIs Console.
     * @param instance Database instance ID. This does not include the project ID.
     * @param content the {@link com.google.api.services.sqladmin.model.DatabaseInstance}
     * @return the request
     */
    public Patch patch(String project, String instance, com.google.api.services.sqladmin.model.DatabaseInstance content) throws java.io.IOException {
      Patch result = new Patch(project, instance, content);
      initialize(result);
      return result;
    }

    public class Patch extends SQLAdminRequest<com.google.api.services.sqladmin.model.InstancesUpdateResponse> {

      private static final String REST_PATH = "projects/{project}/instances/{instance}";

      /**
       * Updates settings of a database instance. Caution: This is not a partial update, so you must
       * include values for all the settings that you want to retain. For partial updates, use patch..
       * This method supports patch semantics.
       *
       * Create a request for the method "instances.patch".
       *
       * This request holds the parameters needed by the the sqladmin server.  After setting any
       * optional parameters, call the {@link Patch#execute()} method to invoke the remote operation.
       * <p> {@link Patch#initialize(AbstractGoogleClientRequest)} must be called to initialize this
       * instance immediately after invoking the constructor. </p>
       *
       * @param project Project ID of the project that contains the instance. You can find this on the project summary page
     *        of the Google APIs Console.
       * @param instance Database instance ID. This does not include the project ID.
       * @param content the {@link com.google.api.services.sqladmin.model.DatabaseInstance}
       * @since 1.13
       */
      protected Patch(String project, String instance, com.google.api.services.sqladmin.model.DatabaseInstance content) {
        super(SQLAdmin.this, "PATCH", REST_PATH, content, com.google.api.services.sqladmin.model.InstancesUpdateResponse.class);
        this.project = Preconditions.checkNotNull(project, "Required parameter project must be specified.");
        this.instance = Preconditions.checkNotNull(instance, "Required parameter instance must be specified.");
      }

      @Override
      public Patch setAlt(String alt) {
        return (Patch) super.setAlt(alt);
      }

      @Override
      public Patch setFields(String fields) {
        return (Patch) super.setFields(fields);
      }

      @Override
      public Patch setKey(String key) {
        return (Patch) super.setKey(key);
      }

      @Override
      public Patch setOauthToken(String oauthToken) {
        return (Patch) super.setOauthToken(oauthToken);
      }

      @Override
      public Patch setPrettyPrint(Boolean prettyPrint) {
        return (Patch) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Patch setQuotaUser(String quotaUser) {
        return (Patch) super.setQuotaUser(quotaUser);
      }

      @Override
      public Patch setUserIp(String userIp) {
        return (Patch) super.setUserIp(userIp);
      }

      /**
     * Project ID of the project that contains the instance. You can find this on the project
     * summary page of the Google APIs Console.
     */
      @com.google.api.client.util.Key
      private String project;

      /** Project ID of the project that contains the instance. You can find this on the project summary page
     of the Google APIs Console.
       */
      public String getProject() {
        return project;
      }

      /**
     * Project ID of the project that contains the instance. You can find this on the project
     * summary page of the Google APIs Console.
     */
      public Patch setProject(String project) {
        this.project = project;
        return this;
      }

      /** Database instance ID. This does not include the project ID. */
      @com.google.api.client.util.Key
      private String instance;

      /** Database instance ID. This does not include the project ID.
       */
      public String getInstance() {
        return instance;
      }

      /** Database instance ID. This does not include the project ID. */
      public Patch setInstance(String instance) {
        this.instance = instance;
        return this;
      }

    }
    /**
     * Restarts a database instance.
     *
     * Create a request for the method "instances.restart".
     *
     * This request holds the parameters needed by the the sqladmin server.  After setting any optional
     * parameters, call the {@link Restart#execute()} method to invoke the remote operation.
     *
     * @param project Project ID of the project that contains the instance to be restarted. You can find this on the
     *        project summary page of the Google APIs Console.
     * @param instance Database instance ID. This does not include the project ID.
     * @return the request
     */
    public Restart restart(String project, String instance) throws java.io.IOException {
      Restart result = new Restart(project, instance);
      initialize(result);
      return result;
    }

    public class Restart extends SQLAdminRequest<com.google.api.services.sqladmin.model.InstancesRestartResponse> {

      private static final String REST_PATH = "projects/{project}/instances/{instance}/restart";

      /**
       * Restarts a database instance.
       *
       * Create a request for the method "instances.restart".
       *
       * This request holds the parameters needed by the the sqladmin server.  After setting any
       * optional parameters, call the {@link Restart#execute()} method to invoke the remote operation.
       * <p> {@link Restart#initialize(AbstractGoogleClientRequest)} must be called to initialize this
       * instance immediately after invoking the constructor. </p>
       *
       * @param project Project ID of the project that contains the instance to be restarted. You can find this on the
     *        project summary page of the Google APIs Console.
       * @param instance Database instance ID. This does not include the project ID.
       * @since 1.13
       */
      protected Restart(String project, String instance) {
        super(SQLAdmin.this, "POST", REST_PATH, null, com.google.api.services.sqladmin.model.InstancesRestartResponse.class);
        this.project = Preconditions.checkNotNull(project, "Required parameter project must be specified.");
        this.instance = Preconditions.checkNotNull(instance, "Required parameter instance must be specified.");
      }

      @Override
      public Restart setAlt(String alt) {
        return (Restart) super.setAlt(alt);
      }

      @Override
      public Restart setFields(String fields) {
        return (Restart) super.setFields(fields);
      }

      @Override
      public Restart setKey(String key) {
        return (Restart) super.setKey(key);
      }

      @Override
      public Restart setOauthToken(String oauthToken) {
        return (Restart) super.setOauthToken(oauthToken);
      }

      @Override
      public Restart setPrettyPrint(Boolean prettyPrint) {
        return (Restart) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Restart setQuotaUser(String quotaUser) {
        return (Restart) super.setQuotaUser(quotaUser);
      }

      @Override
      public Restart setUserIp(String userIp) {
        return (Restart) super.setUserIp(userIp);
      }

      /**
     * Project ID of the project that contains the instance to be restarted. You can find this on
     * the project summary page of the Google APIs Console.
     */
      @com.google.api.client.util.Key
      private String project;

      /** Project ID of the project that contains the instance to be restarted. You can find this on the
     project summary page of the Google APIs Console.
       */
      public String getProject() {
        return project;
      }

      /**
     * Project ID of the project that contains the instance to be restarted. You can find this on
     * the project summary page of the Google APIs Console.
     */
      public Restart setProject(String project) {
        this.project = project;
        return this;
      }

      /** Database instance ID. This does not include the project ID. */
      @com.google.api.client.util.Key
      private String instance;

      /** Database instance ID. This does not include the project ID.
       */
      public String getInstance() {
        return instance;
      }

      /** Database instance ID. This does not include the project ID. */
      public Restart setInstance(String instance) {
        this.instance = instance;
        return this;
      }

    }
    /**
     * Restores a backup of a database instance.
     *
     * Create a request for the method "instances.restoreBackup".
     *
     * This request holds the parameters needed by the the sqladmin server.  After setting any optional
     * parameters, call the {@link RestoreBackup#execute()} method to invoke the remote operation.
     *
     * @param project Project ID of the project that contains the instance. You can find this on the project summary page
     *        of the Google APIs Console.
     * @param instance Database instance ID. This does not include the project ID.
     * @param backupConfiguration The identifier of the backup configuration. This gets generated automatically when a backup
     *        configuration is created.
     * @param dueTime The time when this run is due to start in RFC 3339 format, for example 2012-11-15T16:19:00.094Z.
     * @return the request
     */
    public RestoreBackup restoreBackup(String project, String instance, String backupConfiguration, String dueTime) throws java.io.IOException {
      RestoreBackup result = new RestoreBackup(project, instance, backupConfiguration, dueTime);
      initialize(result);
      return result;
    }

    public class RestoreBackup extends SQLAdminRequest<com.google.api.services.sqladmin.model.InstancesRestoreBackupResponse> {

      private static final String REST_PATH = "projects/{project}/instances/{instance}/restoreBackup";

      /**
       * Restores a backup of a database instance.
       *
       * Create a request for the method "instances.restoreBackup".
       *
       * This request holds the parameters needed by the the sqladmin server.  After setting any
       * optional parameters, call the {@link RestoreBackup#execute()} method to invoke the remote
       * operation. <p> {@link RestoreBackup#initialize(AbstractGoogleClientRequest)} must be called to
       * initialize this instance immediately after invoking the constructor. </p>
       *
       * @param project Project ID of the project that contains the instance. You can find this on the project summary page
     *        of the Google APIs Console.
       * @param instance Database instance ID. This does not include the project ID.
       * @param backupConfiguration The identifier of the backup configuration. This gets generated automatically when a backup
     *        configuration is created.
       * @param dueTime The time when this run is due to start in RFC 3339 format, for example 2012-11-15T16:19:00.094Z.
       * @since 1.13
       */
      protected RestoreBackup(String project, String instance, String backupConfiguration, String dueTime) {
        super(SQLAdmin.this, "POST", REST_PATH, null, com.google.api.services.sqladmin.model.InstancesRestoreBackupResponse.class);
        this.project = Preconditions.checkNotNull(project, "Required parameter project must be specified.");
        this.instance = Preconditions.checkNotNull(instance, "Required parameter instance must be specified.");
        this.backupConfiguration = Preconditions.checkNotNull(backupConfiguration, "Required parameter backupConfiguration must be specified.");
        this.dueTime = Preconditions.checkNotNull(dueTime, "Required parameter dueTime must be specified.");
      }

      @Override
      public RestoreBackup setAlt(String alt) {
        return (RestoreBackup) super.setAlt(alt);
      }

      @Override
      public RestoreBackup setFields(String fields) {
        return (RestoreBackup) super.setFields(fields);
      }

      @Override
      public RestoreBackup setKey(String key) {
        return (RestoreBackup) super.setKey(key);
      }

      @Override
      public RestoreBackup setOauthToken(String oauthToken) {
        return (RestoreBackup) super.setOauthToken(oauthToken);
      }

      @Override
      public RestoreBackup setPrettyPrint(Boolean prettyPrint) {
        return (RestoreBackup) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public RestoreBackup setQuotaUser(String quotaUser) {
        return (RestoreBackup) super.setQuotaUser(quotaUser);
      }

      @Override
      public RestoreBackup setUserIp(String userIp) {
        return (RestoreBackup) super.setUserIp(userIp);
      }

      /**
     * Project ID of the project that contains the instance. You can find this on the project
     * summary page of the Google APIs Console.
     */
      @com.google.api.client.util.Key
      private String project;

      /** Project ID of the project that contains the instance. You can find this on the project summary page
     of the Google APIs Console.
       */
      public String getProject() {
        return project;
      }

      /**
     * Project ID of the project that contains the instance. You can find this on the project
     * summary page of the Google APIs Console.
     */
      public RestoreBackup setProject(String project) {
        this.project = project;
        return this;
      }

      /** Database instance ID. This does not include the project ID. */
      @com.google.api.client.util.Key
      private String instance;

      /** Database instance ID. This does not include the project ID.
       */
      public String getInstance() {
        return instance;
      }

      /** Database instance ID. This does not include the project ID. */
      public RestoreBackup setInstance(String instance) {
        this.instance = instance;
        return this;
      }

      /**
     * The identifier of the backup configuration. This gets generated automatically when a backup
     * configuration is created.
     */
      @com.google.api.client.util.Key
      private String backupConfiguration;

      /** The identifier of the backup configuration. This gets generated automatically when a backup
     configuration is created.
       */
      public String getBackupConfiguration() {
        return backupConfiguration;
      }

      /**
     * The identifier of the backup configuration. This gets generated automatically when a backup
     * configuration is created.
     */
      public RestoreBackup setBackupConfiguration(String backupConfiguration) {
        this.backupConfiguration = backupConfiguration;
        return this;
      }

      /**
     * The time when this run is due to start in RFC 3339 format, for example
     * 2012-11-15T16:19:00.094Z.
     */
      @com.google.api.client.util.Key
      private String dueTime;

      /** The time when this run is due to start in RFC 3339 format, for example 2012-11-15T16:19:00.094Z.
       */
      public String getDueTime() {
        return dueTime;
      }

      /**
     * The time when this run is due to start in RFC 3339 format, for example
     * 2012-11-15T16:19:00.094Z.
     */
      public RestoreBackup setDueTime(String dueTime) {
        this.dueTime = dueTime;
        return this;
      }

    }
    /**
     * Updates settings of a database instance. Caution: This is not a partial update, so you must
     * include values for all the settings that you want to retain. For partial updates, use patch.
     *
     * Create a request for the method "instances.update".
     *
     * This request holds the parameters needed by the the sqladmin server.  After setting any optional
     * parameters, call the {@link Update#execute()} method to invoke the remote operation.
     *
     * @param project Project ID of the project that contains the instance. You can find this on the project summary page
     *        of the Google APIs Console.
     * @param instance Database instance ID. This does not include the project ID.
     * @param content the {@link com.google.api.services.sqladmin.model.DatabaseInstance}
     * @return the request
     */
    public Update update(String project, String instance, com.google.api.services.sqladmin.model.DatabaseInstance content) throws java.io.IOException {
      Update result = new Update(project, instance, content);
      initialize(result);
      return result;
    }

    public class Update extends SQLAdminRequest<com.google.api.services.sqladmin.model.InstancesUpdateResponse> {

      private static final String REST_PATH = "projects/{project}/instances/{instance}";

      /**
       * Updates settings of a database instance. Caution: This is not a partial update, so you must
       * include values for all the settings that you want to retain. For partial updates, use patch.
       *
       * Create a request for the method "instances.update".
       *
       * This request holds the parameters needed by the the sqladmin server.  After setting any
       * optional parameters, call the {@link Update#execute()} method to invoke the remote operation.
       * <p> {@link Update#initialize(AbstractGoogleClientRequest)} must be called to initialize this
       * instance immediately after invoking the constructor. </p>
       *
       * @param project Project ID of the project that contains the instance. You can find this on the project summary page
     *        of the Google APIs Console.
       * @param instance Database instance ID. This does not include the project ID.
       * @param content the {@link com.google.api.services.sqladmin.model.DatabaseInstance}
       * @since 1.13
       */
      protected Update(String project, String instance, com.google.api.services.sqladmin.model.DatabaseInstance content) {
        super(SQLAdmin.this, "PUT", REST_PATH, content, com.google.api.services.sqladmin.model.InstancesUpdateResponse.class);
        this.project = Preconditions.checkNotNull(project, "Required parameter project must be specified.");
        this.instance = Preconditions.checkNotNull(instance, "Required parameter instance must be specified.");
      }

      @Override
      public Update setAlt(String alt) {
        return (Update) super.setAlt(alt);
      }

      @Override
      public Update setFields(String fields) {
        return (Update) super.setFields(fields);
      }

      @Override
      public Update setKey(String key) {
        return (Update) super.setKey(key);
      }

      @Override
      public Update setOauthToken(String oauthToken) {
        return (Update) super.setOauthToken(oauthToken);
      }

      @Override
      public Update setPrettyPrint(Boolean prettyPrint) {
        return (Update) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Update setQuotaUser(String quotaUser) {
        return (Update) super.setQuotaUser(quotaUser);
      }

      @Override
      public Update setUserIp(String userIp) {
        return (Update) super.setUserIp(userIp);
      }

      /**
     * Project ID of the project that contains the instance. You can find this on the project
     * summary page of the Google APIs Console.
     */
      @com.google.api.client.util.Key
      private String project;

      /** Project ID of the project that contains the instance. You can find this on the project summary page
     of the Google APIs Console.
       */
      public String getProject() {
        return project;
      }

      /**
     * Project ID of the project that contains the instance. You can find this on the project
     * summary page of the Google APIs Console.
     */
      public Update setProject(String project) {
        this.project = project;
        return this;
      }

      /** Database instance ID. This does not include the project ID. */
      @com.google.api.client.util.Key
      private String instance;

      /** Database instance ID. This does not include the project ID.
       */
      public String getInstance() {
        return instance;
      }

      /** Database instance ID. This does not include the project ID. */
      public Update setInstance(String instance) {
        this.instance = instance;
        return this;
      }

    }

  }

  /**
   * An accessor for creating requests from the Operations collection.
   *
   * <p>The typical use is:</p>
   * <pre>
   *   {@code SQLAdmin sqladmin = new SQLAdmin(...);}
   *   {@code SQLAdmin.Operations.List request = sqladmin.operations().list(parameters ...)}
   * </pre>
   *
   * @return the resource collection
   */
  public Operations operations() {
    return new Operations();
  }

  /**
   * The "operations" collection of methods.
   */
  public class Operations {

    /**
     * Retrieves an instance operation that has been performed on an instance.
     *
     * Create a request for the method "operations.get".
     *
     * This request holds the parameters needed by the the sqladmin server.  After setting any optional
     * parameters, call the {@link Get#execute()} method to invoke the remote operation.
     *
     * @param project Project ID of the project that contains the instance. You can find this on the project summary page
     *        of the Google APIs Console.
     * @param instance Database instance ID. This does not include the project ID.
     * @param operation Instance operation ID.
     * @return the request
     */
    public Get get(String project, String instance, String operation) throws java.io.IOException {
      Get result = new Get(project, instance, operation);
      initialize(result);
      return result;
    }

    public class Get extends SQLAdminRequest<com.google.api.services.sqladmin.model.InstanceOperation> {

      private static final String REST_PATH = "projects/{project}/instances/{instance}/operations/{operation}";

      /**
       * Retrieves an instance operation that has been performed on an instance.
       *
       * Create a request for the method "operations.get".
       *
       * This request holds the parameters needed by the the sqladmin server.  After setting any
       * optional parameters, call the {@link Get#execute()} method to invoke the remote operation. <p>
       * {@link Get#initialize(AbstractGoogleClientRequest)} must be called to initialize this instance
       * immediately after invoking the constructor. </p>
       *
       * @param project Project ID of the project that contains the instance. You can find this on the project summary page
     *        of the Google APIs Console.
       * @param instance Database instance ID. This does not include the project ID.
       * @param operation Instance operation ID.
       * @since 1.13
       */
      protected Get(String project, String instance, String operation) {
        super(SQLAdmin.this, "GET", REST_PATH, null, com.google.api.services.sqladmin.model.InstanceOperation.class);
        this.project = Preconditions.checkNotNull(project, "Required parameter project must be specified.");
        this.instance = Preconditions.checkNotNull(instance, "Required parameter instance must be specified.");
        this.operation = Preconditions.checkNotNull(operation, "Required parameter operation must be specified.");
      }

      @Override
      public com.google.api.client.http.HttpResponse executeUsingHead() throws java.io.IOException {
        return super.executeUsingHead();
      }

      @Override
      public com.google.api.client.http.HttpRequest buildHttpRequestUsingHead() throws java.io.IOException {
        return super.buildHttpRequestUsingHead();
      }

      @Override
      public Get setAlt(String alt) {
        return (Get) super.setAlt(alt);
      }

      @Override
      public Get setFields(String fields) {
        return (Get) super.setFields(fields);
      }

      @Override
      public Get setKey(String key) {
        return (Get) super.setKey(key);
      }

      @Override
      public Get setOauthToken(String oauthToken) {
        return (Get) super.setOauthToken(oauthToken);
      }

      @Override
      public Get setPrettyPrint(Boolean prettyPrint) {
        return (Get) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Get setQuotaUser(String quotaUser) {
        return (Get) super.setQuotaUser(quotaUser);
      }

      @Override
      public Get setUserIp(String userIp) {
        return (Get) super.setUserIp(userIp);
      }

      /**
     * Project ID of the project that contains the instance. You can find this on the project
     * summary page of the Google APIs Console.
     */
      @com.google.api.client.util.Key
      private String project;

      /** Project ID of the project that contains the instance. You can find this on the project summary page
     of the Google APIs Console.
       */
      public String getProject() {
        return project;
      }

      /**
     * Project ID of the project that contains the instance. You can find this on the project
     * summary page of the Google APIs Console.
     */
      public Get setProject(String project) {
        this.project = project;
        return this;
      }

      /** Database instance ID. This does not include the project ID. */
      @com.google.api.client.util.Key
      private String instance;

      /** Database instance ID. This does not include the project ID.
       */
      public String getInstance() {
        return instance;
      }

      /** Database instance ID. This does not include the project ID. */
      public Get setInstance(String instance) {
        this.instance = instance;
        return this;
      }

      /** Instance operation ID. */
      @com.google.api.client.util.Key
      private String operation;

      /** Instance operation ID.
       */
      public String getOperation() {
        return operation;
      }

      /** Instance operation ID. */
      public Get setOperation(String operation) {
        this.operation = operation;
        return this;
      }

    }
    /**
     * Lists all instance operations that have been performed on the given database instance in the
     * reverse chronological order of the start time.
     *
     * Create a request for the method "operations.list".
     *
     * This request holds the parameters needed by the the sqladmin server.  After setting any optional
     * parameters, call the {@link List#execute()} method to invoke the remote operation.
     *
     * @param project Project ID of the project that contains the instance. You can find this on the project summary page
     *        of the Google APIs Console.
     * @param instance Database instance ID. This does not include the project ID.
     * @return the request
     */
    public List list(String project, String instance) throws java.io.IOException {
      List result = new List(project, instance);
      initialize(result);
      return result;
    }

    public class List extends SQLAdminRequest<com.google.api.services.sqladmin.model.OperationsListResponse> {

      private static final String REST_PATH = "projects/{project}/instances/{instance}/operations";

      /**
       * Lists all instance operations that have been performed on the given database instance in the
       * reverse chronological order of the start time.
       *
       * Create a request for the method "operations.list".
       *
       * This request holds the parameters needed by the the sqladmin server.  After setting any
       * optional parameters, call the {@link List#execute()} method to invoke the remote operation. <p>
       * {@link List#initialize(AbstractGoogleClientRequest)} must be called to initialize this instance
       * immediately after invoking the constructor. </p>
       *
       * @param project Project ID of the project that contains the instance. You can find this on the project summary page
     *        of the Google APIs Console.
       * @param instance Database instance ID. This does not include the project ID.
       * @since 1.13
       */
      protected List(String project, String instance) {
        super(SQLAdmin.this, "GET", REST_PATH, null, com.google.api.services.sqladmin.model.OperationsListResponse.class);
        this.project = Preconditions.checkNotNull(project, "Required parameter project must be specified.");
        this.instance = Preconditions.checkNotNull(instance, "Required parameter instance must be specified.");
      }

      @Override
      public com.google.api.client.http.HttpResponse executeUsingHead() throws java.io.IOException {
        return super.executeUsingHead();
      }

      @Override
      public com.google.api.client.http.HttpRequest buildHttpRequestUsingHead() throws java.io.IOException {
        return super.buildHttpRequestUsingHead();
      }

      @Override
      public List setAlt(String alt) {
        return (List) super.setAlt(alt);
      }

      @Override
      public List setFields(String fields) {
        return (List) super.setFields(fields);
      }

      @Override
      public List setKey(String key) {
        return (List) super.setKey(key);
      }

      @Override
      public List setOauthToken(String oauthToken) {
        return (List) super.setOauthToken(oauthToken);
      }

      @Override
      public List setPrettyPrint(Boolean prettyPrint) {
        return (List) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public List setQuotaUser(String quotaUser) {
        return (List) super.setQuotaUser(quotaUser);
      }

      @Override
      public List setUserIp(String userIp) {
        return (List) super.setUserIp(userIp);
      }

      /**
     * Project ID of the project that contains the instance. You can find this on the project
     * summary page of the Google APIs Console.
     */
      @com.google.api.client.util.Key
      private String project;

      /** Project ID of the project that contains the instance. You can find this on the project summary page
     of the Google APIs Console.
       */
      public String getProject() {
        return project;
      }

      /**
     * Project ID of the project that contains the instance. You can find this on the project
     * summary page of the Google APIs Console.
     */
      public List setProject(String project) {
        this.project = project;
        return this;
      }

      /** Database instance ID. This does not include the project ID. */
      @com.google.api.client.util.Key
      private String instance;

      /** Database instance ID. This does not include the project ID.
       */
      public String getInstance() {
        return instance;
      }

      /** Database instance ID. This does not include the project ID. */
      public List setInstance(String instance) {
        this.instance = instance;
        return this;
      }

      /** Maximum number of operations per response. */
      @com.google.api.client.util.Key
      private Long maxResults;

      /** Maximum number of operations per response.
       */
      public Long getMaxResults() {
        return maxResults;
      }

      /** Maximum number of operations per response. */
      public List setMaxResults(Long maxResults) {
        this.maxResults = maxResults;
        return this;
      }

      /** A previously-returned page token representing part of the larger set of results to view. */
      @com.google.api.client.util.Key
      private String pageToken;

      /** A previously-returned page token representing part of the larger set of results to view.
       */
      public String getPageToken() {
        return pageToken;
      }

      /** A previously-returned page token representing part of the larger set of results to view. */
      public List setPageToken(String pageToken) {
        this.pageToken = pageToken;
        return this;
      }

    }

  }

  /**
   * An accessor for creating requests from the Tiers collection.
   *
   * <p>The typical use is:</p>
   * <pre>
   *   {@code SQLAdmin sqladmin = new SQLAdmin(...);}
   *   {@code SQLAdmin.Tiers.List request = sqladmin.tiers().list(parameters ...)}
   * </pre>
   *
   * @return the resource collection
   */
  public Tiers tiers() {
    return new Tiers();
  }

  /**
   * The "tiers" collection of methods.
   */
  public class Tiers {

    /**
     * Lists all available service tiers for Google Cloud SQL, for example D1, D2. For related
     * information, see Pricing.
     *
     * Create a request for the method "tiers.list".
     *
     * This request holds the parameters needed by the the sqladmin server.  After setting any optional
     * parameters, call the {@link List#execute()} method to invoke the remote operation.
     *
     * @return the request
     */
    public List list() throws java.io.IOException {
      List result = new List();
      initialize(result);
      return result;
    }

    public class List extends SQLAdminRequest<com.google.api.services.sqladmin.model.TiersListResponse> {

      private static final String REST_PATH = "tiers";

      /**
       * Lists all available service tiers for Google Cloud SQL, for example D1, D2. For related
       * information, see Pricing.
       *
       * Create a request for the method "tiers.list".
       *
       * This request holds the parameters needed by the the sqladmin server.  After setting any
       * optional parameters, call the {@link List#execute()} method to invoke the remote operation. <p>
       * {@link List#initialize(AbstractGoogleClientRequest)} must be called to initialize this instance
       * immediately after invoking the constructor. </p>
       *
       * @since 1.13
       */
      protected List() {
        super(SQLAdmin.this, "GET", REST_PATH, null, com.google.api.services.sqladmin.model.TiersListResponse.class);
      }

      @Override
      public com.google.api.client.http.HttpResponse executeUsingHead() throws java.io.IOException {
        return super.executeUsingHead();
      }

      @Override
      public com.google.api.client.http.HttpRequest buildHttpRequestUsingHead() throws java.io.IOException {
        return super.buildHttpRequestUsingHead();
      }

      @Override
      public List setAlt(String alt) {
        return (List) super.setAlt(alt);
      }

      @Override
      public List setFields(String fields) {
        return (List) super.setFields(fields);
      }

      @Override
      public List setKey(String key) {
        return (List) super.setKey(key);
      }

      @Override
      public List setOauthToken(String oauthToken) {
        return (List) super.setOauthToken(oauthToken);
      }

      @Override
      public List setPrettyPrint(Boolean prettyPrint) {
        return (List) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public List setQuotaUser(String quotaUser) {
        return (List) super.setQuotaUser(quotaUser);
      }

      @Override
      public List setUserIp(String userIp) {
        return (List) super.setUserIp(userIp);
      }

    }

  }

  /**
   * Builder for {@link SQLAdmin}.
   *
   * <p>
   * Implementation is not thread-safe.
   * </p>
   *
   * @since 1.3.0
   */
  public static final class Builder extends AbstractGoogleJsonClient.Builder {

    /**
     * Returns an instance of a new builder.
     *
     * @param transport HTTP transport
     * @param jsonFactory JSON factory
     * @param httpRequestInitializer HTTP request initializer or {@code null} for none
     * @since 1.7
     */
    public Builder(HttpTransport transport, JsonFactory jsonFactory,
        HttpRequestInitializer httpRequestInitializer) {
      super(
          transport,
          jsonFactory,
          DEFAULT_ROOT_URL,
          DEFAULT_SERVICE_PATH,
          httpRequestInitializer,
          false);
    }

    /** Builds a new instance of {@link SQLAdmin}. */
    @Override
    public SQLAdmin build() {
      return new SQLAdmin(getTransport(),
          getHttpRequestInitializer(),
          getRootUrl(),
          getServicePath(),
          getObjectParser(),
          getGoogleClientRequestInitializer(),
          getApplicationName(),
          getSuppressPatternChecks());
    }

    @Override
    public Builder setRootUrl(String rootUrl) {
      return (Builder) super.setRootUrl(rootUrl);
    }

    @Override
    public Builder setServicePath(String servicePath) {
      return (Builder) super.setServicePath(servicePath);
    }

    @Override
    public Builder setHttpRequestInitializer(HttpRequestInitializer httpRequestInitializer) {
      return (Builder) super.setHttpRequestInitializer(httpRequestInitializer);
    }

    @Override
    public Builder setApplicationName(String applicationName) {
      return (Builder) super.setApplicationName(applicationName);
    }

    @Override
    public Builder setSuppressPatternChecks(boolean suppressPatternChecks) {
      return (Builder) super.setSuppressPatternChecks(suppressPatternChecks);
    }

    /**
     * Set the {@link SQLAdminRequestInitializer}.
     *
     * @since 1.12
     */
    public Builder setSQLAdminRequestInitializer(
        SQLAdminRequestInitializer sqladminRequestInitializer) {
      return (Builder) super.setGoogleClientRequestInitializer(sqladminRequestInitializer);
    }

    @Override
    public Builder setGoogleClientRequestInitializer(
        GoogleClientRequestInitializer googleClientRequestInitializer) {
      return (Builder) super.setGoogleClientRequestInitializer(googleClientRequestInitializer);
    }
  }
}
